// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AICS32/samples/irq/irq0.c $
// $Rev: 50491 $
// $Date: 2022-02-22 18:01:34 -0600 (Tue, 22 Feb 2022) $

// 16AICS32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _init(int fd)
{
	int	errs;
	int	ret;

	ret		= aics32_ioctl(fd, AICS32_IOCTL_INITIALIZE, NULL);
	errs	= ret ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _irq0_initialize_test(int fd)
{
	int	errs	= 0;

	gsc_label("Initialize");
	errs	+= irq_wait_test(fd, AICS32_WAIT_GSC_INIT_DONE, _init);
	return(errs);
}



//*****************************************************************************
static int _auto_cal(int fd)
{
	int	errs	= 0;
	int	ret;

	ret		= aics32_ioctl(fd, AICS32_IOCTL_INITIALIZE, NULL);
	errs	+= ret ? 1 : 0;
	ret		= aics32_ioctl(fd, AICS32_IOCTL_AUTO_CALIBRATE, NULL);
	errs	+= ret ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _irq0_auto_calibrate_test(int fd)
{
	int	errs;

	gsc_label("Auto-Calibrate");
	errs	= irq_wait_test(fd, AICS32_WAIT_GSC_AUTO_CAL_DONE, _auto_cal);
	return(errs);
}



//*****************************************************************************
static int _scan_start(int fd)
{
	int	errs	= 0;
	int	ret;
	s32	val;

	ret		= aics32_ioctl(fd, AICS32_IOCTL_INITIALIZE, NULL);
	errs	+= ret ? 1 : 0;

	val		= AICS32_IRQ0_SCAN_START;
	ret		= aics32_ioctl(fd, AICS32_IOCTL_IRQ0_SEL, &val);
	errs	+= ret ? 1 : 0;

	val		= AICS32_AI_CLK_SRC_BCR;
	ret		= aics32_ioctl(fd, AICS32_IOCTL_AI_CLK_SRC, &val);
	errs	+= ret ? 1 : 0;

	ret		= aics32_ioctl(fd, AICS32_IOCTL_INPUT_SYNC, NULL);
	errs	+= ret ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _irq0_scan_start_test(int fd)
{
	int	errs;

	gsc_label("Scan Start");
	errs	= irq_wait_test(fd, AICS32_WAIT_GSC_SCAN_START, _scan_start);
	return(errs);
}



//*****************************************************************************
static int _scan_done(int fd)
{
	int	errs	= 0;
	int	ret;
	s32	val;

	ret		= aics32_ioctl(fd, AICS32_IOCTL_INITIALIZE, NULL);
	errs	+= ret ? 1 : 0;

	val		= AICS32_IRQ0_SCAN_DONE;
	ret		= aics32_ioctl(fd, AICS32_IOCTL_IRQ0_SEL, &val);
	errs	+= ret ? 1 : 0;

	val		= AICS32_AI_CLK_SRC_BCR;
	ret		= aics32_ioctl(fd, AICS32_IOCTL_AI_CLK_SRC, &val);
	errs	+= ret ? 1 : 0;

	ret		= aics32_ioctl(fd, AICS32_IOCTL_INPUT_SYNC, NULL);
	errs	+= ret ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _irq0_scan_done_test(int fd)
{
	int	errs;

	gsc_label("Scan Done");
	errs	= irq_wait_test(fd, AICS32_WAIT_GSC_SCAN_DONE, _scan_done);
	return(errs);
}



//*****************************************************************************
int irq0_test(int fd)
{
	int	errs	= 0;

	gsc_label("IRQ0");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _irq0_initialize_test(fd);
	errs	+= _irq0_auto_calibrate_test(fd);
	errs	+= _irq0_scan_start_test(fd);
	errs	+= _irq0_scan_done_test(fd);

	gsc_label_level_dec();
	return(errs);
}



