// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AICS32/samples/regs/main.c $
// $Rev: 52043 $
// $Date: 2022-11-22 10:24:15 -0600 (Tue, 22 Nov 2022) $

// 16AICS32: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	s32	_index			= 0;



//*****************************************************************************
static int _parse_args(int argc, char** argv)
{
	static const gsc_arg_item_t	list[]	=
	{
		//	type				var				arg			values	desc
		{	GSC_ARG_DEV_INDEX,	&_index,		"index",	{ 1 },	"The zero based index of the device to access."	}
	};

	static const gsc_arg_set_t	set	=
	{
		/* name	*/	"regs",
		/* desc	*/	"Register Access Application",
		/* qty	*/	SIZEOF_ARRAY(list),
		/* list	*/	list
	};

	int	errs;

	errs	= gsc_args_parse(argc, argv, &set);

	return(errs);
}



//*****************************************************************************
static void _show_access_index(int index)
{
	gsc_label("Accessing Device");
	printf("%d\n", index);
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	int		errs	= 0;
	int		fd		= 0;
	int		qty		= 0;
	int		ret;

	for (;;)
	{
		gsc_label_init(28);
		errs	= _parse_args(argc, argv);

		if (errs)
			break;

		// Introduction

		os_id_host();
		_show_time_stamp();
		_show_access_index(_index);
		errs	+= aics32_init_util(1);
		errs	+= os_id_driver(aics32_open, aics32_read, aics32_close);
		errs	+= aics32_count_boards(1, &qty);

		if ((errs) || (qty <= 0))
			break;

		// Open access to device

		errs	= aics32_open_util(_index, 0, -1, 1, &fd);

		if (errs == 0)
		{
			// Device identification

			errs	= aics32_id_device(fd, -1, 1);

			// Perform testing

			errs	+= perform_tests(fd);

			// Close device access

			errs	+= aics32_close_util(fd, -1, 1);
		}

		// End of processing.

		break;
	}

	ret	= ((qty <= 0) || (errs)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}



