// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AICS32/samples/rxrate/perform.c $
// $Rev: 52044 $
// $Date: 2022-11-22 10:33:04 -0600 (Tue, 22 Nov 2022) $

// 16AICS32: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_100K	(100L * 1024)



// variables ******************************************************************

static u32	_rx[_100K];



//*****************************************************************************
static int _rate_test(const args_t* args, long get)
{
	struct timeval	begin;
	int				errs	= 0;
	long			got;
	long long		limit	= (long long) args->rx_mb * 1000000L;
	struct timeval	minimum;
	struct timeval	now;
	long long		sps;
	long long		total	= 0;
	long			us		= 1000000L * args->seconds;

	gsc_label("Reading");

	gettimeofday(&begin, NULL);
	minimum.tv_sec	= begin.tv_sec + args->seconds;
	minimum.tv_usec	= begin.tv_usec;

	for (;;)
	{
		got	= aics32_read(args->fd, _rx, get);

		if (got < 0)
		{
			errs++;
			printf("FAIL <---  (read");
			printf(", total so far %lld", total);
			printf(", requested %ld bytes", (long) get);
			printf(", error %ld)\n", (long) got);
			break;
		}

		gettimeofday(&now, NULL);
		total	+= got;

		if ((total >= limit) && (now.tv_sec > minimum.tv_sec))
		{
			us	= (now.tv_sec - begin.tv_sec) * 1000000L
				+ (now.tv_usec - begin.tv_usec);
			sps	= (total * 10000000L / 4 / us + 5) / 10;

			printf(	"PASS  (");
			gsc_label_long_comma(total);
			printf(" Bytes, %ld.%06ld Seconds, ", us / 1000000, us % 1000000);
			gsc_label_long_comma(sps);
			printf(" S/S)\n");
			break;
		}
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs	= 0;
	long	get		= sizeof(_rx);

	errs	+= aics32_config_ai(args->fd, -1, 1, AICS32_SCAN_SIZE_0_31, 3125);
	errs	+= aics32_rx_io_mode(args->fd, -1, 1, args->io_mode, NULL);

	gsc_label ("Read Rate");
	printf("(%d second minimum, %d MB minimum)\n", (int) args->seconds, (int) args->rx_mb);
	gsc_label_level_inc();

	errs	+= aics32_ai_buf_clear(args->fd, -1, 1);

	errs	+= _rate_test(args, get);

	gsc_label_level_dec();
	return(errs);
}



